@@ -106,6 +106,7 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
 
     m_view = new DolphinView(url, this);
     connect(m_view, SIGNAL(urlChanged(KUrl)),                   m_urlNavigator, SLOT(setUrl(KUrl)));
+    connect(m_view, SIGNAL(urlChanged(KUrl)),                   m_messageWidget, SLOT(hide()));
     connect(m_view, SIGNAL(writeStateChanged(bool)),            this, SIGNAL(writeStateChanged(bool)));
     connect(m_view, SIGNAL(requestItemInfo(KFileItem)),         this, SLOT(showItemInfo(KFileItem)));
     connect(m_view, SIGNAL(itemActivated(KFileItem)),           this, SLOT(slotItemActivated(KFileItem)));
@@ -128,6 +129,8 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
             this, SLOT(slotUrlNavigatorLocationChanged(KUrl)));
     connect(m_urlNavigator, SIGNAL(historyChanged()),
             this, SLOT(slotHistoryChanged()));
+    connect(m_urlNavigator, SIGNAL(returnPressed()),
+            this, SLOT(slotReturnPressed()));
 
     // Initialize status bar
     m_statusBar = new DolphinStatusBar(this);
@@ -156,8 +159,10 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
             this, SLOT(setNameFilter(QString)));
     connect(m_filterBar, SIGNAL(closeRequest()),
             this, SLOT(closeFilterBar()));
+    connect(m_filterBar, SIGNAL(focusViewRequest()),
+            this, SLOT(requestFocus()));
     connect(m_view, SIGNAL(urlChanged(KUrl)),
-            m_filterBar, SLOT(clear()));
+            m_filterBar, SLOT(slotUrlChanged()));
 
     m_topLayout->addWidget(m_urlNavigator);
     m_topLayout->addWidget(m_searchBox);
@@ -477,37 +482,12 @@ void DolphinViewContainer::slotItemActivated(const KFileItem& item)
     // results in an active view.
     m_view->setActive(true);
 
-    KUrl url = item.targetUrl();
-
-    if (item.isDir()) {
+    const KUrl& url = DolphinView::openItemAsFolderUrl(item, GeneralSettings::browseThroughArchives());
+    if (!url.isEmpty()) {
         m_view->setUrl(url);
         return;
     }
 
-    if (GeneralSettings::browseThroughArchives() && item.isFile() && url.isLocalFile()) {
-        // Generic mechanism for redirecting to tar:/<path>/ when clicking on a tar file,
-        // zip:/<path>/ when clicking on a zip file, etc.
-        // The .protocol file specifies the mimetype that the kioslave handles.
-        // Note that we don't use mimetype inheritance since we don't want to
-        // open OpenDocument files as zip folders...
-        const QString protocol = KProtocolManager::protocolForArchiveMimetype(item.mimetype());
-        if (!protocol.isEmpty()) {
-            url.setProtocol(protocol);
-            m_view->setUrl(url);
-            return;
-        }
-    }
-
-    if (item.mimetype() == QLatin1String("application/x-desktop")) {
-        // Redirect to the URL in Type=Link desktop files
-        KDesktopFile desktopFile(url.toLocalFile());
-        if (desktopFile.hasLinkType()) {
-            url = desktopFile.readUrl();
-            m_view->setUrl(url);
-            return;
-        }
-    }
-
     item.run();
 }
 
@@ -531,8 +511,7 @@ void DolphinViewContainer::showItemInfo(const KFileItem& item)
 
 void DolphinViewContainer::closeFilterBar()
 {
-    m_filterBar->hide();
-    m_filterBar->clear();
+    m_filterBar->closeFilterBar();
     m_view->setFocus();
     emit showFilterBarChanged(false);
 }
@@ -574,6 +553,8 @@ void DolphinViewContainer::slotUrlNavigatorLocationAboutToBeChanged(const KUrl&
 
 void DolphinViewContainer::slotUrlNavigatorLocationChanged(const KUrl& url)
 {
+    slotReturnPressed();
+
     if (KProtocolManager::supportsListing(url)) {
         setSearchModeEnabled(isSearchUrl(url));
         m_view->setUrl(url);
@@ -616,7 +597,8 @@ void DolphinViewContainer::slotUrlNavigatorLocationChanged(const KUrl& url)
 
 void DolphinViewContainer::dropUrls(const KUrl& destination, QDropEvent* event)
 {
-    const QString error = DragAndDropHelper::dropUrls(KFileItem(), destination, event);
+    QString error;
+    DragAndDropHelper::dropUrls(KFileItem(), destination, event, error);
     if (!error.isEmpty()) {
         showMessage(error, Error);
     }
@@ -657,6 +639,13 @@ void DolphinViewContainer::slotHistoryChanged()
     }
 }
 
+void DolphinViewContainer::slotReturnPressed()
+{
+    if (!GeneralSettings::editableUrl()) {
+        m_urlNavigator->setUrlEditable(false);
+    }
+}
+
 void DolphinViewContainer::startSearching()
 {
     const KUrl url = m_searchBox->urlForSearching();
